<?php

namespace Drp\AdfsSocialiteProvider;

use Illuminate\Support\Arr;
use Laravel\Socialite\Two\AbstractProvider;
use Laravel\Socialite\Two\InvalidStateException;
use Laravel\Socialite\Two\ProviderInterface;
use Laravel\Socialite\Two\User;
use Lcobucci\JWT\Parser;

class AdfsProvider extends AbstractProvider implements ProviderInterface
{
    /**
     * The separating character for the requested scopes.
     *
     * @var string
     */
    protected $scopeSeparator = ' ';

    /**
     * The scopes being requested.
     *
     * @var string[]
     */
    protected $scopes = [
        'openid',
        'profile',
        'email',
    ];

    /**
     * {@inheritdoc}
     */
    protected function getAuthUrl($state)
    {
        return $this->buildAuthUrlFromBase(config('turnstile.auth.authorization_url'), $state);
    }

    /**
     * {@inheritdoc}
     */
    protected function getTokenUrl()
    {
        return config('turnstile.auth.token_url');
    }

    /**
     * Get the POST fields for the token request.
     *
     * @param string $code
     * @return string[]
     */
    // phpcs:disable SlevomatCodingStandard.TypeHints.TypeHintDeclaration.MissingParameterTypeHint
    // phpcs:disable SlevomatCodingStandard.TypeHints.TypeHintDeclaration.MissingReturnTypeHint
    protected function getTokenFields($code)
    {
        //phpcs:enable
        return Arr::add(
            parent::getTokenFields($code),
            'grant_type',
            'authorization_code'
        );
    }

    /**
     * {@inheritdoc}
     */
    public function user()
    {
        if ($this->hasInvalidState()) {
            throw new InvalidStateException;
        }

        $response = $this->getAccessTokenResponse($this->getCode());

        $user = $this->mapUserToObject($this->getUserByToken(
            $token = Arr::get($response, 'id_token', Arr::get($response, 'access_token'))
        ));

        return $user->setToken(Arr::get($response, 'access_token'))
            ->setRefreshToken(Arr::get($response, 'refresh_token'))
            ->setExpiresIn(Arr::get($response, 'expires_in'))
            ->map(['id_token' => $token]);
    }

    /**
     * {@inheritdoc}
     */
    protected function getUserByToken($token)
    {
        $decoded = (new Parser)->parse($token);

        return collect($decoded->getClaims())
            ->map->getValue()
            ->toArray();
    }

    /**
     * {@inheritdoc}
     */
    protected function mapUserToObject(array $user)
    {
        $name = Arr::get($user, 'given_name') . ' ' . Arr::get($user, 'family_name');

        return (new User)->setRaw($user)
            ->map([
                'id' => $user['upn'],
                'name' => $name,
                'nickname' => Arr::get($user, 'given_name'),
            ])
            ->map(Arr::only($user, config('turnstile.auth.claims')));
    }
}
