<?php

namespace Drp\AdfsSocialiteProvider\Http\Controllers;

use GuzzleHttp\Exception\ClientException;
use Illuminate\Http\RedirectResponse;
use Laravel\Socialite\AbstractUser as SocialiteUser;
use Laravel\Socialite\Facades\Socialite;

class HandleProviderCallback
{
    public function __invoke(): RedirectResponse
    {
        try {
            $user = Socialite::driver('adfs')->user();
        } catch (ClientException $exception) {
            return redirect()->to('/')->withError('Unable to login. Please try again later.');
        }

        $authUser = $this->findOrCreateUser($user);

        auth()->login($authUser, true);

        return app('adfs.redirect');
    }

    /**
     * Return user if exists; create and return if doesn't
     */
    private function findOrCreateUser(SocialiteUser $adfsUser)
    {
        $userModel = config('turnstile.auth.user_model');

        $user = $userModel::where('adfs_id', $adfsUser->id)->first();

        if ($user === null) {
            $user = $userModel::create([
                'name' => $adfsUser->given_name . ' ' . $adfsUser->family_name,
                'email' => $adfsUser->email,
                'adfs_id' => $adfsUser->id,
            ]);
        }

        if (method_exists($user, 'mapAdfsUserAttributes')) {
            $user->mapAdfsUserAttributes($adfsUser);
        }

        return $user;
    }
}
